-- Marajoara_Functions
-- Author: Jarcast
-- DateCreated: 8/30/2024 10:04:01 AM
--==========================================================================================================================
include("FLuaVector.lua")
--==========================================================================================================================
-- GLOBALS
--==========================================================================================================================
local Teams 					= Teams
local activeiPlayer				= Game.GetActivePlayer()
local activePlayer				= Players[activeiPlayer]
local activeTeamID				= activePlayer:GetTeam()
local activeTeam				= Teams[activeTeamID]
--==========================================================================================================================
-- UTILITIES
--==========================================================================================================================
-- UTILITIES
----------------------------------------------------------------------------------------------------------------------------
--Game_IsUsingMod
local CBPmodID = "d1b6328c-ff44-4b0d-aad7-c657f83610cd"
local CBOmodID = "8411a7a8-dad3-4622-a18e-fcc18324c799"
local CSDmodID = "eead0050-1e3f-4178-a91f-26cf1881ac39"
local EEraModID = "ce8aa614-7ef7-4a45-a179-5329869e8d6d"
local EECBPmodID  = "87b2deae-83cb-9b88-9017-79e741d3739c"
local JPEPmodID  = "6f6c20b3-020c-4c62-9668-38e87eb66065"

function Game_IsUsingMod(modID)
	for _, mod in pairs(Modding.GetActivatedMods()) do
		if mod.ID == modID then
			return true
		end
	end
	return false
end

local isUsingEEra = Game_IsUsingMod(EEraModID) or Game_IsUsingMod(EECBPmodID)
local isUsingCBP  = Game_IsUsingMod(CBPmodID)
local isUsingCBO  = Game_IsUsingMod(CBOmodID)
local isUsingJPE  = Game_IsUsingMod(JPEPmodID)
----------------------------------------------------------------------------------------------------------------------------
--HasTrait
function HasTrait(player, traitID)
	if Game_IsUsingMod(CBPmodID) then 
		return player:HasTrait(traitID)
	else
		local leaderType = GameInfo.Leaders[player:GetLeaderType()].Type
		local traitType  = GameInfo.Traits[traitID].Type
		for row in GameInfo.Leader_Traits("LeaderType = '" .. leaderType .. "' AND TraitType = '" .. traitType .. "'") do
			return true
		end
	end
	return false
end
-------------------------------------------------------------------------------------------------------------------------
-- GetRandom
function GetRandom(lower, upper)
    return Game.Rand((upper + 1) - lower, "") + lower
end
-------------------------------------------------------------------------------------------------------------------------
--IsTraitActive
local slotComputerID = SlotStatus["SS_COMPUTER"]
local slotTakenID = SlotStatus["SS_TAKEN"]
function IsTraitActive(traitID)
	for slotID = 0, GameDefines.MAX_MAJOR_CIVS-1, 1 do
		local slotPlayer = Players[slotID]
		local slotStatus = PreGame.GetSlotStatus(slotID)
		if (slotStatus == slotTakenID or slotStatus == slotComputerID) then
			if HasTrait(slotPlayer, traitID) then
				return true
			end
		end
	end
	return false
end
-------------------------------------------------------------------------------------------------------------------------
function Neirai_GetNearestCity(pPlayer, pPlot)
    local distance = 9999
    local cNearestCity = nil
    for cCity in pPlayer:Cities() do
        local pCityPlot = cCity:Plot()
        local between = Map.PlotDistance(pCityPlot:GetX(), pCityPlot:GetY(), pPlot:GetX(), pPlot:GetY())
        if between < distance then
            distance = between
            cNearestCity = cCity
        end
    end
    return cNearestCity
end
-------------------------------------------------------------------------------------------------------------------------
function GetTechRevealIDFromResID(resourceID)
	for row in DB.Query("SELECT b.ID TechID from Resources a, Technologies b WHERE a.TechReveal = b.Type and a.ID = '" .. resourceID .. "'") do
		return row.TechID
	end
end
--==========================================================================================================================
-- UNIQUE FUNCTIONS
--==========================================================================================================================
-- GLOBALS
--------------------------------------------------------------------------------------------------------------------------
local onlycoastal= GameDefines.LAKE_MAX_AREA_SIZE +1
local CityWorkingRadius = GameDefines.MAXIMUM_WORK_PLOT_DISTANCE or 3
local iMod = ((GameInfo.GameSpeeds[Game.GetGameSpeedType()].BuildPercent)/100)
local iCiv = GameInfoTypes.CIVILIZATION_JAR_MARAJOARA
local traitMarajoaraID = GameInfoTypes.TRAIT_JAR_MARAJOARA
local g_IsTraitActive = IsTraitActive(traitMarajoaraID)
local iMboiaPromo = GameInfoTypes.PROMOTION_JAR_MBOIA
local fForest = GameInfoTypes.FEATURE_FOREST
local fJungle = GameInfoTypes.FEATURE_JUNGLE
local iTeso = GameInfoTypes.BUILDING_JAR_TESO 
local iTesoDummy2 = GameInfoTypes.BUILDING_JAR_TESO_D2 
local iTesoDummy3 = GameInfoTypes.BUILDING_JAR_TESO_D3 
local iTesoDummy4 = GameInfoTypes.BUILDING_JAR_TESO_D4 
local iTesoDummy5 = GameInfoTypes.BUILDING_JAR_TESO_D5 
local iTesoDummy6 = GameInfoTypes.BUILDING_JAR_TESO_D6 
local iTesoDummy7 = GameInfoTypes.BUILDING_JAR_TESO_D7 
local iTesoDummy8 = GameInfoTypes.BUILDING_JAR_TESO_D8 
local iTesoDummy9 = GameInfoTypes.BUILDING_JAR_TESO_D9
------------------------------------------------------------------------------------------------------------------------
-- QUICK MOD BALANCING PANEL
------------------------------------------------------------------------------------------------------------------------
-- Here are the changeable values that determine the strength of the effects described in the design.
-- UA
local EraTurnsThreshold = 0.95

-- UU Snake-Canoe
local iDrainedHPperUnit = 5
--========================================================================================================================
-- FUNCTIONS
--========================================================================================================================
-- MARAJOARA UA
------------------------------------------------------------------------------------------------------------------------
local tAllowedTerrain = {}
tAllowedTerrain[GameInfoTypes.TERRAIN_GRASS] = true
tAllowedTerrain[GameInfoTypes.TERRAIN_PLAINS] = true
tAllowedTerrain[GameInfoTypes.TERRAIN_TUNDRA] = true

function Jar_MarajoaraUANewEra(eTeam, eraID, bFirst)
	for playerID, pPlayer in pairs(Players) do
		if pPlayer:IsEverAlive() and pPlayer:GetTeam() == eTeam and HasTrait(pPlayer, traitMarajoaraID) then
			for pCity in pPlayer:Cities() do
				local iForestTiles = 0
				local iJungleTiles = 0
				for i = 0, pCity:GetNumCityPlots()-1, 1 do
					local pPlot = pCity:GetCityIndexPlot(i)
					if pPlot then
						if pPlot:GetFeatureType() == fForest then iForestTiles= iForestTiles+1 end
						if pPlot:GetFeatureType() == fJungle then iJungleTiles= iJungleTiles+1 end
					end
				end
				for i = 0, pCity:GetNumCityPlots()-1, 1 do
					local pPlot = pCity:GetCityIndexPlot(i)
					if pPlot and pPlot:GetOwner() == playerID and (not pPlot:IsWater()) then
						if pPlot:GetImprovementType() ~= -1 then
							Game.SetPlotExtraYield(pPlot:GetX(), pPlot:GetY(), GameInfo.Yields.YIELD_FOOD.ID, 1 )
							if pPlayer:IsHuman() then
								local sMessage = "+1 [ICON_FOOD]"
								Events.AddPopupTextEvent(HexToWorld(ToHexFromGrid(Vector2(pPlot:GetX(), pPlot:GetY()))), sMessage, 3)
							end
						end
						if pPlot:IsBeingWorked() and tAllowedTerrain[pPlot:GetTerrainType()] and pPlot:GetFeatureType() == -1 and (not pPlot:IsMountain()) and (not pPlot:IsCity()) then
							if iForestTiles > iJungleTiles or pPlot:GetTerrainType() == GameInfoTypes.TERRAIN_TUNDRA then
								pPlot:SetFeatureType(fForest)
								if pPlayer:IsHuman() then
									local sMessage = "+Forest"
									Events.AddPopupTextEvent(HexToWorld(ToHexFromGrid(Vector2(pPlot:GetX(), pPlot:GetY()))), sMessage, 5)
								end
							else
								pPlot:SetFeatureType(fJungle)
								if pPlayer:IsHuman() then
									local sMessage = "+Jungle"
									Events.AddPopupTextEvent(HexToWorld(ToHexFromGrid(Vector2(pPlot:GetX(), pPlot:GetY()))), sMessage, 5)
								end
							end
						end
					end
				end
			end
		end
	end
end
if g_IsTraitActive then GameEvents.TeamSetEra.Add(Jar_MarajoaraUANewEra) end
--========================================================================================================================
-- UI - MALOCA
--========================================================================================================================
local tBuildRemove = {}
tBuildRemove[GameInfoTypes.BUILD_REMOVE_JUNGLE] = true
tBuildRemove[GameInfoTypes.BUILD_REMOVE_FOREST] = true

function Jar_CanBuildMaloca(iPlayer, eUnit, iX, iY, eBuild)
	local pPlot = Map.GetPlot(iX, iY)
	local pPlayer = Players[iPlayer]
	local ResID = pPlot:GetResourceType()
	local bHasTechReveal = Teams[pPlayer:GetTeam()]:IsHasTech( GetTechRevealIDFromResID( ResID ) )
    if eBuild == GameInfoTypes.BUILD_JAR_MALOCA then
		if ResID < 0 then
			return false
		end
		if pPlot:GetFeatureType() ~= GameInfoTypes.FEATURE_JUNGLE and pPlot:GetFeatureType() ~= GameInfoTypes.FEATURE_FOREST then
			return false
		end
	end
    if eBuild == GameInfoTypes.BUILD_PLANTATION and HasTrait(pPlayer, traitMarajoaraID) then
		if pPlot:GetFeatureType() == GameInfoTypes.FEATURE_JUNGLE or pPlot:GetFeatureType() == GameInfoTypes.FEATURE_FOREST then
			return false
		end
	end
    if tBuildRemove[eBuild] and ResID > -1 and HasTrait(pPlayer, traitMarajoaraID) then
		return (not bHasTechReveal)
	end
	return true
end
GameEvents.PlayerCanBuild.Add(Jar_CanBuildMaloca)
--========================================================================================================================
-- UU - SNAKE CANOE
--========================================================================================================================
local iNumDirections = DirectionTypes.NUM_DIRECTION_TYPES - 1
function SnakeCanoe_HealthDrain(playerID)
	local pPlayer = Players[playerID]	
	if pPlayer:GetNumUnitPromotions(iMboiaPromo) <1 then return end
	local pTeam = Teams[pPlayer:GetTeam()]
	for pUnit in pPlayer:Units() do
		if pUnit:IsHasPromotion(iMboiaPromo) then
			local pPlot = pUnit:GetPlot()
			local iDrainQty = 0
			for iDirection = 0, iNumDirections, 1 do
				local pAdjPlot = Map.PlotDirection(pPlot:GetX(), pPlot:GetY(), iDirection)
				local pAdjUnit = pAdjPlot:GetUnit(0)
				if pAdjUnit then
					local adjUnitTeamID = Players[pAdjUnit:GetOwner()]:GetTeam()
					if pTeam:IsAtWar(adjUnitTeamID) then
						pAdjUnit:ChangeDamage(iDrainedHPperUnit)
						iDrainQty = (iDrainQty - iDrainedHPperUnit) --we want this quantity to be negative
					end
				end
			end
			if iDrainQty<0 then
				pUnit:ChangeDamage(iDrainQty)
				if pPlayer:IsHuman() then
					local sMessage = "Drain: +" .. (iDrainQty * -1) .. " HP"
					Events.AddPopupTextEvent(HexToWorld(ToHexFromGrid(Vector2(pPlot:GetX(), pPlot:GetY()))), sMessage, 1)
				end
			end
		end
	end
end
if g_IsTraitActive then GameEvents.PlayerDoTurn.Add(SnakeCanoe_HealthDrain) end
--========================================================================================================================
-- UB - TESO
--========================================================================================================================
function Jar_TesoRequirements(iHexX, iHexY, newPopulation, citySize)
	local pPlot = Map.GetPlot(ToGridFromHex(iHexX, iHexY))
	local pCity = pPlot:GetPlotCity()
    local pPlayer = Players[pCity:GetOwner()]
	if not pPlayer:IsAlive() then return end
	if pCity:IsHasBuilding(iTeso) then
		if newPopulation >= 5 and not pCity:IsHasBuilding(iTesoDummy2) then
			pCity:SetNumRealBuilding(iTesoDummy2, 1)
		elseif newPopulation >= 10 and not pCity:IsHasBuilding(iTesoDummy3) then
			pCity:SetNumRealBuilding(iTesoDummy3, 1)
		elseif newPopulation >= 15 and not pCity:IsHasBuilding(iTesoDummy4) then
			pCity:SetNumRealBuilding(iTesoDummy4, 1)
		elseif newPopulation >= 20 and not pCity:IsHasBuilding(iTesoDummy5) then
			pCity:SetNumRealBuilding(iTesoDummy5, 1)
		elseif newPopulation >= 25 and not pCity:IsHasBuilding(iTesoDummy6) then
			pCity:SetNumRealBuilding(iTesoDummy6, 1)
		elseif newPopulation >= 30 and not pCity:IsHasBuilding(iTesoDummy7) then
			pCity:SetNumRealBuilding(iTesoDummy7, 1)
		elseif newPopulation >= 35 and not pCity:IsHasBuilding(iTesoDummy8) then
			pCity:SetNumRealBuilding(iTesoDummy8, 1)
		elseif newPopulation >= 40 and not pCity:IsHasBuilding(iTesoDummy9) then
			pCity:SetNumRealBuilding(iTesoDummy9, 1)
        end
    end
end
Events.SerialEventCityPopulationChanged.Add(Jar_TesoRequirements)
---------------------------------------------------------
print("Jarcast's Marajoara functions loaded")
--==========================================================================================================================
--==========================================================================================================================